@@ -1,5 +1,9 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
 
 /* ts A71019 */
 
@@ -19,6 +23,9 @@
 #define Libburn_detach_done_workeR 1
 */
 
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
 
 #include "libburn.h"
 #include "transport.h"
@@ -36,6 +43,7 @@
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
+#include <signal.h>
 
 /*
 #include <a ssert.h>
@@ -135,7 +143,7 @@ static void add_worker(int w_type, struct burn_drive *d,
 	pthread_attr_t attr;
 #endif
 
-	a = malloc(sizeof(struct w_list));
+	a = calloc(1, sizeof(struct w_list));
 	a->w_type = w_type;
 	a->drive = d;
 	a->u = *(union w_list_data *)data;
@@ -175,11 +183,6 @@ static void remove_worker(pthread_t th)
 {
 	struct w_list *a, *l = NULL;
 
-#ifdef Libburn_detach_done_workeR
-	int ret;
-	char msg[80];
-#endif
-
 	for (a = workers; a; l = a, a = a->next)
 		if (a->thread == th) {
 			if (l)
@@ -192,8 +195,12 @@ static void remove_worker(pthread_t th)
 			/* Alternative : threads get detached and thus should
 					dispose themselves.
 			*/
-			ret = pthread_detach(th);
+			pthread_detach(th);
 /*
+			int ret;
+			char msg[80];
+
+			ret = pthread_detach(th);
 			sprintf(msg,
 			 "remove_workers(): pid= %lu  pthread_detach(%lu)= %d",
 			 (unsigned long) getpid(), (unsigned long) th, ret);
@@ -228,6 +235,22 @@ static void *scan_worker_func(struct w_list *w)
 	return NULL;
 }
 
+static void reset_progress(struct burn_drive *d, int sessions, int tracks,
+				int indices, int sectors, int flag)
+{
+	/* reset the progress indicator */
+	d->progress.session = 0;
+	d->progress.sessions = sessions;
+	d->progress.track = 0;
+	d->progress.tracks = tracks;
+	d->progress.index = 0;
+	d->progress.indices = indices;
+	d->progress.start_sector = 0;
+	d->progress.sectors = sectors;
+	d->progress.sector = 0;
+}
+
+
 int burn_drive_scan(struct burn_drive_info *drives[], unsigned int *n_drives)
 {
 	struct scan_opts o;
@@ -297,8 +320,27 @@ drive_is_active:;
 
 static void *erase_worker_func(struct w_list *w)
 {
+
+#define Libburn_protect_erase_threaD 1
+
+#ifdef Libburn_protect_erase_threaD
+	sigset_t sigset, oldset;
+
+	/* Protect blank thread from being interrupted by external signals */
+	sigfillset(&sigset);
+	sigdelset(&sigset, SIGSEGV);
+	sigdelset(&sigset, SIGILL);
+	pthread_sigmask(SIG_SETMASK, &sigset, &oldset);
+#endif /* Libburn_protect_erase_threaD */
+
 	burn_disc_erase_sync(w->u.erase.drive, w->u.erase.fast);
 	remove_worker(pthread_self());
+
+#ifdef Libburn_protect_erase_threaD
+	/* (just in case it would not end with all signals blocked) */
+	pthread_sigmask(SIG_SETMASK, &oldset, NULL);
+#endif /* Libburn_protect_erase_threaD */
+
 	return NULL;
 }
 
@@ -310,8 +352,9 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	/* a ssert(drive); */
 	/* a ssert(!SCAN_GOING()); */
 	/* a ssert(!find_worker(drive)); */
+
 	if((drive == NULL)) {
-		libdax_msgs_submit(libdax_messenger, drive->global_index,
+		libdax_msgs_submit(libdax_messenger, -1,
 			0x00020104,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 			"NULL pointer caught in burn_disc_erase", 0, 0);
@@ -325,6 +368,9 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 			0, 0);
 		return;
 	}
+
+	reset_progress(drive, 1, 1, 1, 0x10000, 0);
+
 	/* A70103 : will be set to 0 by burn_disc_erase_sync() */
 	drive->cancel = 1;
 
@@ -336,7 +382,9 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	/* ts A70909 : the willingness to burn any BURN_DISC_FULL media is
 	               inappropriate. One would rather need a -force option
 	               Note: keep this in sync with mmc_read_disc_info() */
-	if ((drive->current_profile != 0x0a &&
+	/* ts B10321 : Allowed role 5 to be blanked */
+	if ((drive->drive_role == 1 &&
+	     drive->current_profile != 0x0a &&
 	     drive->current_profile != 0x13 &&
 	     drive->current_profile != 0x14 &&
 	     drive->status != BURN_DISC_FULL)
@@ -345,13 +393,18 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 	     drive->status != BURN_DISC_APPENDABLE &&
 	     drive->status != BURN_DISC_BLANK)
 	    ||
-	    (drive->drive_role != 1)
+	    (drive->drive_role != 1 && drive->drive_role != 5)
 	   ) {
+		char msg[160];
+
+		sprintf(msg, "Drive and media state unsuitable for blanking. (role= %d , profile= 0x%x , status= %d)",
+			drive->drive_role,
+			(unsigned int) drive->current_profile,
+			drive->status);
 		libdax_msgs_submit(libdax_messenger, drive->global_index,
 			0x00020130,
 			LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
-			"Drive and media state unsuitable for blanking",
-			0, 0);
+			msg, 0, 0);
 		return;
 	}
 
@@ -365,9 +418,28 @@ void burn_disc_erase(struct burn_drive *drive, int fast)
 /* ts A61230 */
 static void *format_worker_func(struct w_list *w)
 {
+
+#define Libburn_protect_format_threaD 1
+
+#ifdef Libburn_protect_format_threaD
+	sigset_t sigset, oldset;
+
+	/* Protect format thread from being interrupted by external signals */
+	sigfillset(&sigset);
+	sigdelset(&sigset, SIGSEGV);
+	sigdelset(&sigset, SIGILL);
+	pthread_sigmask(SIG_SETMASK, &sigset, &oldset);
+#endif /* Libburn_protect_format_threaD */
+
 	burn_disc_format_sync(w->u.format.drive, w->u.format.size,
 				w->u.format.flag);
 	remove_worker(pthread_self());
+
+#ifdef Libburn_protect_format_threaD
+	/* (just in case it would not end with all signals blocked) */
+	pthread_sigmask(SIG_SETMASK, &oldset, NULL);
+#endif /* Libburn_protect_format_threaD */
+
 	return NULL;
 }
 
@@ -377,7 +449,9 @@ void burn_disc_format(struct burn_drive *drive, off_t size, int flag)
 {
 	struct format_opts o;
 	int ok = 0, ret;
-	char msg[160];
+	char msg[40];
+
+	reset_progress(drive, 1, 1, 1, 0x10000, 0);
 
 	if ((SCAN_GOING()) || find_worker(drive) != NULL) {
 		libdax_msgs_submit(libdax_messenger, drive->global_index,
@@ -449,42 +523,6 @@ void burn_disc_format(struct burn_drive *drive, off_t size, int flag)
 			return;
 
 		} 
-		if ((flag & 6) != 6 || (flag & 128)) {
-		    if ((flag & 64) && !(drive->current_feat23h_byte4 & 2)) {
-			if (drive->current_feat23h_byte4 & 1) {
-				libdax_msgs_submit(libdax_messenger,
-					drive->global_index, 0x00020165,
-				   	LIBDAX_MSGS_SEV_WARNING,
-					LIBDAX_MSGS_PRIO_HIGH,
-				      "Drive does not support fast formatting",
-					0, 0);
-				flag &= ~64;
-			} else {
-no_non_default_bd_re:;
-				libdax_msgs_submit(libdax_messenger,
-					drive->global_index, 0x00020167,
-				   	LIBDAX_MSGS_SEV_SORRY,
-					LIBDAX_MSGS_PRIO_HIGH,
-				"Drive does not support non-default formatting",
-					0, 0);
-				drive->cancel = 1;
-				return;
-			}
-		    }
-		    if ((!(flag & 64)) && !(drive->current_feat23h_byte4 & 1)){
-			if (drive->current_feat23h_byte4 & 2) {
-				libdax_msgs_submit(libdax_messenger,
-					drive->global_index, 0x00020166,
-				   	LIBDAX_MSGS_SEV_WARNING,
-					LIBDAX_MSGS_PRIO_HIGH,
-				      "Drive does not support full formatting",
-					0, 0);
-				flag |= 64;
-			} else
-				goto no_non_default_bd_re;
-		    }
-		}
-
 	}
 
 	if (!ok) {
@@ -508,8 +546,22 @@ no_non_default_bd_re:;
 static void *write_disc_worker_func(struct w_list *w)
 {
 	struct burn_drive *d = w->u.write.drive;
+	char msg[80];
+
+#define Libburn_protect_write_threaD 1
+
+#ifdef Libburn_protect_write_threaD
+	sigset_t sigset, oldset;
+
+	/* Protect write thread from being interrupted by external signals */
+	sigfillset(&sigset);
+	sigdelset(&sigset, SIGSEGV);
+	sigdelset(&sigset, SIGILL);
+	pthread_sigmask(SIG_SETMASK, &sigset, &oldset);
+#endif /* Libburn_protect_write_threaD */
 
 	d->thread_pid = getpid();
+	d->thread_tid = pthread_self();
 	d->thread_pid_valid= 1;
 	burn_disc_write_sync(w->u.write.opts, w->u.write.disc);
 	d->thread_pid_valid= 0;
@@ -519,15 +571,28 @@ static void *write_disc_worker_func(struct w_list *w)
 	 */
 	burn_write_opts_free(w->u.write.opts);
 
+	sprintf(msg, "Write thread on drive %d ended", d->global_index);
+	libdax_msgs_submit(libdax_messenger, d->global_index, 0x00020178,
+			LIBDAX_MSGS_SEV_DEBUG, LIBDAX_MSGS_PRIO_HIGH,
+			msg, 0, 0);
+
 	remove_worker(pthread_self());
+	d->busy = BURN_DRIVE_IDLE;
+
+#ifdef Libburn_protect_write_threaD
+	/* (just in case it would not end with all signals blocked) */
+	pthread_sigmask(SIG_SETMASK, &oldset, NULL);
+#endif /* Libburn_protect_write_threaD */
+
 	return NULL;
 }
 
 void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 {
 	struct write_opts o;
-	char reasons[BURN_REASONS_LEN+80];
+	char *reasons= NULL;
 	struct burn_drive *d;
+	int mvalid;
 
 	d = opts->drive;
 
@@ -543,12 +608,20 @@ void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 		return;
 	}
 
+	reset_progress(d, disc->sessions, disc->session[0]->tracks,
+			 disc->session[0]->track[0]->indices, 0, 0);
+
 	/* For the next lines any return indicates failure */
 	d->cancel = 1;
 
 	/* ts A70203 : people have been warned in API specs */
-	if (opts->write_type == BURN_WRITE_NONE)
+	if (opts->write_type == BURN_WRITE_NONE) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x0002017c,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"No valid write type selected", 0, 0);
 		return;
+	}
 
 	if (d->drive_role == 0) {
 		libdax_msgs_submit(libdax_messenger, d->global_index,
@@ -557,19 +630,35 @@ void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 			"Drive is a virtual placeholder (null-drive)", 0, 0);
 		return;
 	}
+	if (d->drive_role == 4) {
+		libdax_msgs_submit(libdax_messenger, d->global_index,
+			0x00020181,
+			LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+			"Pseudo-drive is a read-only file. Cannot write.",
+			0, 0);
+		return;
+	}
 
 	/* ts A61007 : obsolete Assert in spc_select_write_params() */
-	if (d->drive_role == 1 && !d->mdata->valid) {
-		libdax_msgs_submit(libdax_messenger,
+	if (d->drive_role == 1) {
+		mvalid = 0;
+		if (d->mdata != NULL)
+			if (d->mdata->valid > 0)
+				mvalid = 1;
+		if (!mvalid) {
+			libdax_msgs_submit(libdax_messenger,
 				d->global_index, 0x00020113,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 				"Drive capabilities not inquired yet", 0, 0);
-		return;
+			return;
+		}
 	}
 
 	/* ts A70219 : intended to replace all further tests here and many
 	               tests in burn_*_write_sync()
 	*/
+
+        BURN_ALLOC_MEM_VOID(reasons, char, BURN_REASONS_LEN + 80);
 	strcpy(reasons, "Write job parameters are unsuitable:\n");
 	if (burn_precheck_write(opts, disc, reasons + strlen(reasons), 1)
 	     <= 0) {
@@ -577,8 +666,9 @@ void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 				d->global_index, 0x00020139,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 				reasons, 0, 0);
-		return;
+		goto ex;
 	}
+        BURN_FREE_MEM(reasons); reasons= NULL;
 
 	/* ts A90106 : early catching of unformatted BD-RE */
 	if (d->current_profile == 0x43) 
@@ -602,6 +692,9 @@ void burn_disc_write(struct burn_write_opts *opts, struct burn_disc *disc)
 
 	add_worker(Burnworker_type_writE, d,
 			(WorkerFunc) write_disc_worker_func, &o);
+
+ex:;
+	BURN_FREE_MEM(reasons);
 }
 
 
@@ -609,12 +702,30 @@ static void *fifo_worker_func(struct w_list *w)
 {
 	int old;
 
+#define Libburn_protect_fifo_threaD 1
+
+#ifdef Libburn_protect_fifo_threaD
+	sigset_t sigset, oldset;
+
+	/* Protect fifo thread from being interrupted by external signals */
+	sigfillset(&sigset);
+	sigdelset(&sigset, SIGSEGV);
+	sigdelset(&sigset, SIGILL);
+	pthread_sigmask(SIG_SETMASK, &sigset, &oldset);
+#endif /* Libburn_protect_fifo_threaD */
+
 	pthread_setcancelstate(PTHREAD_CANCEL_ENABLE, &old);
 	pthread_setcanceltype(PTHREAD_CANCEL_ASYNCHRONOUS, &old);
 		/* Note: Only burn_fifo_abort() shall cancel the fifo thread */
 
 	burn_fifo_source_shoveller(w->u.fifo.source, w->u.fifo.flag);
 	remove_worker(pthread_self());
+
+#ifdef Libburn_protect_fifo_threaD
+	/* (just in case it would not end with all signals blocked) */
+	pthread_sigmask(SIG_SETMASK, &oldset, NULL);
+#endif /* Libburn_protect_fifo_threaD */
+
 	return NULL;
 }
 
@@ -627,7 +738,8 @@ int burn_fifo_start(struct burn_source *source, int flag)
 	fs->is_started = -1;
 
 	/* create and set up ring buffer */;
-	fs->buf = calloc(fs->chunksize, fs->chunks);
+	fs->buf = burn_os_alloc_buffer(
+			((size_t) fs->chunksize) * (size_t) fs->chunks, 0);
 	if (fs->buf == NULL) {
 		/* >>> could not start ring buffer */;
 		return -1;
@@ -638,6 +750,7 @@ int burn_fifo_start(struct burn_source *source, int flag)
 	add_worker(Burnworker_type_fifO, NULL,
 			(WorkerFunc) fifo_worker_func, &o);
 	fs->is_started = 1;
+
 	return 1;
 }
 
