@@ -1,7 +1,19 @@
+
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
+#ifdef HAVE_CONFIG_H
+#include "../config.h"
+#endif
+
 #include "libburn.h"
 #include "options.h"
 #include "drive.h"
 #include "transport.h"
+#include "init.h"
+#include "write.h"
 
 /* ts A61007 */
 /* #include <a ssert.h> */
@@ -17,7 +29,7 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 {
 	struct burn_write_opts *opts;
 
-	opts = malloc(sizeof(struct burn_write_opts));
+	opts = calloc(1, sizeof(struct burn_write_opts));
 	if (opts == NULL) {
 		libdax_msgs_submit(libdax_messenger, -1, 0x00020111,
 			LIBDAX_MSGS_SEV_FATAL, LIBDAX_MSGS_PRIO_HIGH,
@@ -34,11 +46,22 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 	opts->underrun_proof = drive->mdata->underrun_proof;
 	opts->perform_opc = 1;
 	opts->obs = -1;
+
+#ifdef Libburn_dvd_always_obs_paD
+	opts->obs_pad = 1;
+#else
 	opts->obs_pad = 0;
+#endif
+
 	opts->start_byte = -1;
 	opts->fill_up_media = 0;
 	opts->force_is_set = 0;
 	opts->do_stream_recording = 0;
+	opts->dvd_obs_override = 0;
+	opts->stdio_fsync_size = Libburn_stdio_fsync_limiT;
+	opts->text_packs = NULL;
+	opts->num_text_packs = 0;
+	opts->no_text_pack_crc_check = 0;
 	opts->has_mediacatalog = 0;
 	opts->format = BURN_CDROM;
 	opts->multi = 0;
@@ -48,15 +71,18 @@ struct burn_write_opts *burn_write_opts_new(struct burn_drive *drive)
 
 void burn_write_opts_free(struct burn_write_opts *opts)
 {
-	if (--opts->refcount <= 0)
-		free(opts);
+	if (--opts->refcount > 0)
+		return;
+	if (opts->text_packs != NULL)
+		free(opts->text_packs);
+	free(opts);
 }
 
 struct burn_read_opts *burn_read_opts_new(struct burn_drive *drive)
 {
 	struct burn_read_opts *opts;
 
-	opts = malloc(sizeof(struct burn_read_opts));
+	opts = calloc(1, sizeof(struct burn_read_opts));
 	opts->drive = drive;
 	opts->refcount = 1;
 	opts->raw = 0;
@@ -67,6 +93,7 @@ struct burn_read_opts *burn_read_opts_new(struct burn_drive *drive)
 	opts->report_recovered_errors = 0;
 	opts->transfer_damaged_blocks = 0;
 	opts->hardware_error_retries = 3;
+	opts->dap_bit = 0;
 
 	return opts;
 }
@@ -113,7 +140,7 @@ void burn_write_opts_set_toc_entries(struct burn_write_opts *opts, int count,
 				     struct burn_toc_entry *toc_entries)
 {
 	opts->toc_entries = count;
-	opts->toc_entry = malloc(count * sizeof(struct burn_toc_entry));
+	opts->toc_entry = calloc(count, sizeof(struct burn_toc_entry));
 	memcpy(opts->toc_entry, &toc_entries,
 	       sizeof(struct burn_toc_entry) * count);
 }
@@ -143,7 +170,7 @@ int burn_write_opts_set_simulate(struct burn_write_opts *opts, int sim)
 int burn_write_opts_set_underrun_proof(struct burn_write_opts *opts,
 				       int underrun_proof)
 {
-	if (!opts->drive->mdata->valid)
+	if (opts->drive->mdata->valid <= 0)
 		return 0;
 	if (opts->drive->mdata->underrun_proof) {
 		opts->underrun_proof = underrun_proof;
@@ -166,7 +193,7 @@ void burn_write_opts_set_has_mediacatalog(struct burn_write_opts *opts,
 void burn_write_opts_set_mediacatalog(struct burn_write_opts *opts,
 				      unsigned char mediacatalog[13])
 {
-	memcpy(opts->mediacatalog, &mediacatalog, 13);
+	memcpy(opts->mediacatalog, mediacatalog, 13);
 }
 
 
@@ -177,6 +204,64 @@ void burn_write_opts_set_multi(struct burn_write_opts *opts, int multi)
 }
 
 
+/* ts B11204 */
+/* @param flag bit0=do not verify checksums
+               bit1= repair mismatching checksums
+               bit2= repair checksums if they are 00 00 with each pack
+*/
+int burn_write_opts_set_leadin_text(struct burn_write_opts *opts,
+                                    unsigned char *text_packs,
+                                    int num_packs, int flag)
+{
+	int ret;
+	unsigned char *pack_buffer = NULL;
+
+	if (num_packs > Libburn_leadin_cdtext_packs_maX ) {
+		libdax_msgs_submit(libdax_messenger, opts->drive->global_index,
+				0x0002018b,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				"Too many CD-TEXT packs", 0, 0);
+		ret= 0; goto ex;
+	}
+
+	if (num_packs > 0)
+		BURN_ALLOC_MEM(pack_buffer, unsigned char, num_packs * 18);
+	
+	if (opts->text_packs != NULL) {
+		free(opts->text_packs);
+		opts->text_packs = NULL;
+	}
+
+	if (flag & 1) {
+		opts->no_text_pack_crc_check = 1;
+	} else {
+		opts->no_text_pack_crc_check = 0;
+		ret = burn_cdtext_crc_mismatches(text_packs, num_packs,
+							(flag >> 1) & 3);
+		if (ret > 0) {
+			libdax_msgs_submit(libdax_messenger, -1, 0x0002018f,
+				LIBDAX_MSGS_SEV_FAILURE, LIBDAX_MSGS_PRIO_HIGH,
+				"CD-TEXT pack CRC mismatch", 0, 0);
+			ret = 0; goto ex;
+		} else if (ret < 0) {
+			libdax_msgs_submit(libdax_messenger, -1, 0x00020190,
+				LIBDAX_MSGS_SEV_WARNING, LIBDAX_MSGS_PRIO_HIGH,
+			    "CD-TEXT pack CRC mismatch had to be corrected",
+				0, 0);
+		}
+	}
+
+	if (num_packs > 0) {
+		memcpy(pack_buffer, text_packs, num_packs * 18);
+		opts->text_packs = pack_buffer;
+	}
+	opts->num_text_packs = num_packs;
+	ret = 1;
+ex:;
+	return ret;
+}
+
+
 /* ts A61222 */
 void burn_write_opts_set_start_byte(struct burn_write_opts *opts, off_t value)
 {
@@ -284,6 +369,10 @@ do_sao:;
 	{wt = BURN_WRITE_SAO; goto ex;}
 no_sao:;
 try_tao:;
+	if (opts->num_text_packs > 0) {
+		strcat(reasons, "CD-TEXT: write type SAO required, ");
+		{wt = BURN_WRITE_NONE; goto ex;}
+	}
 	if ((flag & 1) && opts->write_type != BURN_WRITE_TAO)
 		goto try_raw;
 	reason_pt = reasons + strlen(reasons);
@@ -389,6 +478,32 @@ void burn_write_opts_set_stream_recording(struct burn_write_opts *opts,
 }
 
 
+/* ts A91115: API */
+void burn_write_opts_set_dvd_obs(struct burn_write_opts *opts, int obs)
+{
+	if (obs != 0 && obs != 32 * 1024 && obs != 64 * 1024)
+		return;
+	opts->dvd_obs_override = obs;
+}
+
+
+/* ts B20406: API */
+void burn_write_opts_set_obs_pad(struct burn_write_opts *opts, int pad)
+{
+	opts->obs_pad = 2 * !!pad;
+}
+
+
+/* ts A91115: API */
+void burn_write_opts_set_stdio_fsync(struct burn_write_opts *opts, int rythm)
+{
+	if (rythm == -1)
+		opts->stdio_fsync_size = 0;
+	else if (rythm == 0)
+		opts->stdio_fsync_size = Libburn_stdio_fsync_limiT;
+	else if (rythm >= 32)
+		opts->stdio_fsync_size = rythm;
+}
 
 
 /* ts A70901: API */
