@@ -1,3 +1,9 @@
+
+/* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
+   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Provided under GPL version 2 or later.
+*/
+
 #ifndef BURN__OPTIONS_H
 #define BURN__OPTIONS_H
 
@@ -36,7 +42,9 @@ struct burn_write_opts
 	/* ts A61219 : Output block size to trigger buffer flush if hit.
 			 -1 with CD, 32 kB with DVD */
 	int obs;
-	int obs_pad; /* 1=pad up last block to obs */
+	int obs_pad; /* >0 pad up last block to obs, 0 do not
+	                2 indicates burn_write_opts_set_obs_pad(,1)
+	             */
 
 	/* ts A61222 : Start address for media which allow a choice */
 	off_t start_byte;
@@ -49,6 +57,25 @@ struct burn_write_opts
 	*/
 	int force_is_set;
 
+	/* ts A80412 : whether to use WRITE12 with Streaming bit set
+	   rather than WRITE10. Speeds up DVD-RAM. Might help with BD-RE.
+	   This gets transferred to burn_drive.do_stream_recording
+	*/
+	int do_stream_recording;
+
+	/* ts A91115 : override value for .obs on DVD media.
+	   Only values 0, 32K and 64K are allowed for now. */
+	int dvd_obs_override;
+
+	/* ts A91115 : size of the fsync() interval for stdio writing.
+	   Values 0 or >= 32 counted in 2 KB blocks. */
+	int stdio_fsync_size;
+
+	/* ts B11203 : CD-TEXT */
+	unsigned char *text_packs;
+	int num_text_packs;
+	int no_text_pack_crc_check;
+
 	/** A disc can have a media catalog number */
 	int has_mediacatalog;
 	unsigned char mediacatalog[13];
@@ -59,6 +86,18 @@ struct burn_write_opts
 	unsigned char multi;
 };
 
+/* Default value for burn_write_opts.stdio_flush_size
+*/
+#define Libburn_stdio_fsync_limiT 8192
+
+/* Maximum number of Lead-in text packs.
+   READ TOC/PMA/ATIP can at most return 3640.7 packs.
+   The sequence counters of the packs have 8 bits. There are 8 blocks at most.
+   Thus max 2048 packs. 
+ */
+#define Libburn_leadin_cdtext_packs_maX 2048
+
+
 /** Options for disc reading operations. This should be created with
     burn_read_opts_new() and freed with burn_read_opts_free(). */
 struct burn_read_opts
@@ -90,6 +129,14 @@ struct burn_read_opts
 	/** The number of retries the hardware should make to correct
 	    errors. */
 	unsigned char hardware_error_retries;
+
+	/* ts B21119 */
+	/* >>> Needs API access */
+	/** Whether to set DAP bit which allows drive to apply
+	    "flaw obscuring mechanisms like audio data mute and interpolate"
+	*/
+	unsigned int dap_bit;
+
 };
 
 #endif /* BURN__OPTIONS_H */
