@@ -1,7 +1,7 @@
 /* -*- indent-tabs-mode: t; tab-width: 8; c-basic-offset: 8; -*- */
 
 /* Copyright (c) 2004 - 2006 Derek Foreman, Ben Jansens
-   Copyright (c) 2006 - 2012 Thomas Schmitt <scdbackup@gmx.net>
+   Copyright (c) 2006 - 2013 Thomas Schmitt <scdbackup@gmx.net>
    Provided under GPL version 2 or later.
 */
 
@@ -33,6 +33,7 @@
 #include "debug.h"
 #include "options.h"
 #include "init.h"
+#include "util.h"
 
 #include "libdax_msgs.h"
 extern struct libdax_msgs *libdax_messenger;
@@ -1003,6 +1004,9 @@ int burn_scsi_setup_drive(struct burn_drive *d, int bus_no, int host_no,
 	/* ts A61106 */
 	d->silent_on_scsi_error = 0;
 
+	/* ts B21023 */
+	d->had_particular_error = 0;
+
 
 	d->idata = calloc(1, sizeof(struct burn_scsi_inquiry_data));
 	d->mdata = calloc(1, sizeof(struct scsi_mode_data));
@@ -1090,6 +1094,12 @@ enum response scsi_error_msg(struct burn_drive *d, unsigned char *sense,
 		else
 			sprintf(msg, "Logical unit is not ready");
 		goto return_retry;
+	case 0x06:
+		if (*ascq == 0)
+			sprintf(msg, "No reference position found");
+		else
+			break;
+		goto return_fail;
 	case 0x08:
 		if (*ascq == 0)
 			sprintf(msg, "Logical unit communication failure");
@@ -1453,10 +1463,20 @@ static char *scsi_command_name(unsigned int c, int flag)
 		return "READ DISC STRUCTURE";
         case 0xb6:
 		return "SET STREAMING";
+        case 0xb9:
+		return "READ CD MSF";
         case 0xbb:
 		return "SET CD SPEED";
         case 0xbe:
 		return "READ CD";
+
+#ifdef Libburn_develop_quality_scaN
+
+        case 0xf3:
+		return "NEC/OPTIARC REPORT ERROR RATE";
+
+#endif /* Libburn_develop_quality_scaN */
+
 	}
 	return "(NOT IN LIBBURN COMMAND LIST)";
 }
@@ -1570,7 +1590,7 @@ int scsi_show_command_reply(unsigned char *opcode, int data_dir,
 	if (data_dir != FROM_DRIVE)
 		return 2;
 	if (opcode[0] == 0x28 || opcode[0] == 0x3C ||
-	    opcode[0] == 0xA8 || opcode[0] == 0xBE) {
+	    opcode[0] == 0xA8 || opcode[0] == 0xB9 || opcode[0] == 0xBE) {
 							/* READ commands */
 		/* >>> report amount of data */;
 
@@ -1630,9 +1650,8 @@ int scsi_log_cmd(struct command *c, void *fp_in, int flag)
 */
 int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
                    int dxfer_len, void *fp_in, unsigned char sense[18],
-		   int sense_len, int duration, int flag)
+		   int sense_len, double duration, int flag)
 {
-	char durtxt[20];
 	FILE *fp = fp_in;
 	int key, asc, ascq, i, l;
 
@@ -1648,20 +1667,18 @@ int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
 			for (i = 0 ; i < l; i++)
 				fprintf(fp, " %2.2X", sense[i]);
 			fprintf(fp, "\n");
-			durtxt[0] = 0;
-			if (!(flag & 2))
-  				sprintf(durtxt, "   (%6d ms)", duration);
 			spc_decode_sense(sense, 0, &key, &asc, &ascq);
-  			fprintf(fp, "+++ key=%X  asc=%2.2Xh  ascq=%2.2Xh%s\n",
+  			fprintf(fp, "+++ key=%X  asc=%2.2Xh  ascq=%2.2Xh\n",
 				(unsigned int) key, (unsigned int) asc,
-				(unsigned int) ascq, durtxt);
+				(unsigned int) ascq);
 		} else {
 			scsi_show_command_reply(opcode, data_dir, data,
 			                        dxfer_len, fp, 0);
-
-			if (!(flag & 2))
-				fprintf(fp,"%6d ms\n", duration);
 		}
+		if (!(flag & 2))
+			fprintf(fp, " %8.f us     [ %.f ]\n",
+			        duration * 1.0e6,
+			        (burn_get_time(0) - lib_start_time) * 1.0e6);
 		if (burn_sg_log_scsi & 4)
 			fflush(fp);
 	}
@@ -1679,16 +1696,19 @@ int scsi_log_reply(unsigned char *opcode, int data_dir, unsigned char *data,
     @param flag  bit0 causes an error message
                  bit1 do not print duration
 */
-int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
-		 int sense_len, int duration, int flag)
+int scsi_log_err(struct burn_drive *d, struct command *c,
+                 void *fp_in, unsigned char sense[18],
+		 int sense_len, int flag)
 {
 	int ret;
 	unsigned char *data = NULL;
 
 	if (c->page != NULL)
 		data = c->page->data;
+
 	ret= scsi_log_reply(c->opcode, c->dir, data, c->dxfer_len ,
-	                    fp_in, sense, sense_len, duration, flag);
+	                    fp_in, sense, sense_len,
+	                    c->end_time - c->start_time, flag);
 	return ret;
 }
 
@@ -1701,15 +1721,15 @@ int scsi_log_err(struct command *c, void *fp_in, unsigned char sense[18],
 */
 int scsi_eval_cmd_outcome(struct burn_drive *d, struct command *c, void *fp,
 			unsigned char *sense, int sense_len,
-			int duration, time_t start_time, int timeout_ms,
+			time_t start_time, int timeout_ms,
 			int loop_count, int flag)
 {
 	enum response outcome;
-	int done = -1, usleep_time, ret;
+	int done = -1, usleep_time;
 	char *msg = NULL;
 
 	if (burn_sg_log_scsi & 3)
-		scsi_log_err(c, fp, sense, sense_len, duration,
+		scsi_log_err(d, c, fp, sense, sense_len,
 				 (sense_len > 0) | (flag & 2));
 	if (sense_len <= 0)
 		{done = 1; goto ex;}
@@ -1732,7 +1752,9 @@ int scsi_eval_cmd_outcome(struct burn_drive *d, struct command *c, void *fp,
 		}
 		if (time(NULL) + usleep_time / 1000000 - start_time >
 		    timeout_ms / 1000 + 1) {
-			BURN_ALLOC_MEM(msg, char, 320);
+			done = -1; /* In case of alloc failure */
+			BURN_ALLOC_MEM_VOID(msg, char, 320);
+			done = 1;
 			sprintf(msg,
 				"Timeout exceed (%d ms). Retry canceled.\n",
 				timeout_ms);
@@ -1740,7 +1762,6 @@ int scsi_eval_cmd_outcome(struct burn_drive *d, struct command *c, void *fp,
 				0x0002018a,
 				LIBDAX_MSGS_SEV_SORRY, LIBDAX_MSGS_PRIO_HIGH,
 				msg, 0, 0);
-			done = 1;
 			goto err_ex;
 		}
 		if (d->cancel)
